/** \file DXMatrix.h
<h2> Direct3D Matrix Stack - handles world matrix states </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Direct3D Matrix Stack
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_MATRIX_STACK__
#define __DX_MATRIX_STACK__

#include "DXInterface.h"

//! Get a copy of the current world matrix
D3DXMATRIX dxGetMatrix();

//! Set the stack to given matrix
void dxSetMatrix( D3DXMATRIX newMatrix );

//! Returns a reference of the current world matrix
D3DXMATRIX *dxTopMatrix();

//! Push current matrix stack
void dxPushMatrix();

//! Pop current matrix stack
void dxPopMatrix();

//! Push new matrix stack
void dxPushSetMatrix( D3DXMATRIX newMatrix );

//! Rotate current matrix stack top around X axis
void dxRotateX( float deg );

//! Rotate current matrix stack top around Y axis
void dxRotateY( float deg );

//! Rotate current matrix stack top around Z axis
void dxRotateZ( float deg );

//! Translate current matrix stack top
void dxTranslate( float x, float y, float z );

//! Scale current matrix stack top
void dxScale(float sx, float sy, float sz);

//! Multiply current matrix stack top by givern matrix
void dxMultiply( D3DXMATRIX &m );

//! Update Direct3D with the current matrix stack top
void dxCommitMatrix();

#endif