/** \file DXRenderPass.h
<h2> Handles Rendering to Off-screen Buffer </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Handles Rendering to Off-screen Buffer
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_POST_PROCESS__
#define __DX_POST_PROCESS__

#include "DXInterface.h"
#include "DXPrimitives.h"
#include "DXMatrix.h"
#include "DXText.h"

#define MAX_POSTPROCESS_RENDERTARGETS 4

//! Structure that handles an off-screen rendering buffer
//  with support for floating point and multiple targets 
struct RenderPass {

    //! Render buffer width
    int targetWidth;
    //! Render buffer height
    int targetHeight;

    //! Render buffer
    LPDIRECT3DTEXTURE9 targetTexture[ MAX_POSTPROCESS_RENDERTARGETS ];
    //! Render Z buffer
    LPDIRECT3DTEXTURE9 targetZTexture[ MAX_POSTPROCESS_RENDERTARGETS ];
    //! Render buffer surface
    LPDIRECT3DSURFACE9 targetTextureSurface[ MAX_POSTPROCESS_RENDERTARGETS ];
    //! Render Z buffer surface
    LPDIRECT3DSURFACE9 targetZTextureSurface[ MAX_POSTPROCESS_RENDERTARGETS ];

    //! Handle to previous render target
    LPDIRECT3DSURFACE9 oldRenderTarget;
    //! Handle to previous render Z target
    LPDIRECT3DSURFACE9 oldRenderZTarget;
    //! Stores previous viewport state
    D3DVIEWPORT9 oldViewport;


    RenderPass();
    ~RenderPass();

    //! Creates a new render buffer using given dimensions and format
    void createTarget(D3DFORMAT format = D3DFMT_A8R8G8B8, int width = d3dWindowWidth, 
        int height = d3dWindowHeight, int index = 0);

    //! Releases all associated memory and render targets
    void releaseTarget();

    //! Sets Direct3D to render to this buffer
    void setAsTarget(int screenIndex = 0, int index = 0);

    //! Restores Direct3D rendering target to saved previous state
    void restoreTarget(int screenIndex = 0);

    //! Gets off-screen buffer texture given index
    LPDIRECT3DTEXTURE9 getTargetTexture(int index = 0);

    //! Gets off-screen buffer texture given index
    LPDIRECT3DTEXTURE9 operator [] (int index);

    //! Gets off-screen buffer texture of index 0
    operator LPDIRECT3DTEXTURE9();

    //! Renders a buffer ( texture NOT automatically set, use dxTexture() )
    void render();

    //! Dump a screen buffer to a file in BMP format
    void dump( const char* fileName, int index = 0 );

};

#endif