/** \file DXSound.h
<h2> Sound functionality using FMOD library </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Sound & DSP functions
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __DX_SOUND__
#define __DX_SOUND__

#include "DXInterface.h"
#include <fmod/fmod.h>

#define DXSTREAM_DEFAULT_VOL 180
#define DXSTREAM_RATE 44100
#define DXSTREAM_NYQUIST 22050

#define DXSOUNDDSP_BANDS 512
#define DXSOUNDDSP_LOW 300.0f
#define DXSOUNDDSP_HIGH 6000.0f

//! An FMOD sound stream manager
class SoundStream {
protected:
    //! Handle to FMOD stream
    FSOUND_STREAM *ptr;

public:
    //! Whether the streamis currently playing
    bool playing;
    //! The channel number the stream is playing on
    int channel;
    //! Whether the stream has been loaded
    bool used;

    //! Volume level
    int vol;


    SoundStream();
    //! Open a stream from file
    void open(char *fileName);
    //! Stop playback of stream
    void stop();
    //! Free stream handle and memory
    void free();
    //! Set volume of playing stream
    void setVol(int vol);
    //! Set frequency of playing stream
    void setFreq(int freq);
    //! Start playback of stream
    void play();
    //! Seek to a time in milliseconds into playing stream
    void seek(int ms);

};

//! Initalise the sound system
void dxSoundSystemInit();

//! Activate or De-activate FFT module
void dxSoundSystemSetFFT( bool active );

//! Get current audio spectrum as a 512 float array
//  Activate FFT module using dxSoundSystemSetFFT() first
float *dxSoundSystemSpectrum();

//! Get current audio spectrum as low, mid, high levels.
//  Activate FFT module using dxSoundSystemSetFFT() first
void dxSoundSystemLMH( float *low, float *mid, float *high );

#endif