/** \file DXPrimitives.cpp
<h2> Direct3D Primitives Rendering - Renders spheres, rings, cubes...etc </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Direct3D Primitives Rendering
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../EngineHeader/DXPrimitives.h"

LPD3DXMESH dxTeapotMesh = NULL;

LPDIRECT3DVERTEXBUFFER9 dxPrimitiveCubeVertexBuffer = NULL;
LPDIRECT3DINDEXBUFFER9  dxPrimitiveCubeIndexBuffer = NULL;
short dxPrimitiveCubeNumIndices = 36;
short dxPrimitiveCubeIndices[] = {
    0, 1, 2,    // Side 1
    2, 1, 3,
    4, 5, 6,    // Side 2
    6, 5, 7,
    8, 9, 10,   // Side 3
    10, 9, 11,
    12, 13, 14, // Side 4
    14, 13, 15,
    16, 17, 18, // Side 5
    18, 17, 19,
    20, 21, 22, // Side 6
    22, 21, 23
};

LPDIRECT3DVERTEXBUFFER9 dxPrimitivePlaneVertexBuffer = NULL;
LPDIRECT3DVERTEXBUFFER9 dxPrimitiveFastQuadVertexBuffer = NULL;

// --------> dxPrimitivesObj Methods

//! Releases the object's assigned vertex buffer memory
void dxPrimitivesObj::release() { vertexBuffer->Release(); }

// --------> Functions

//! Initialises the global primitive vertexbuffers and other info
void dxPrimitiveInit() {
    // -----> TEAPOT
    D3DXCreateTeapot(d3dDevice, &dxTeapotMesh, NULL);
    // -----> CUBE
    // --> Create cube
    d3dDevice->CreateVertexBuffer( 24 * sizeof(DXVertex), 0, DXVERTEXFVF, D3DPOOL_MANAGED, &dxPrimitiveCubeVertexBuffer, NULL);
    if(!dxPrimitiveCubeVertexBuffer) { wError("dxPrimitiveInit vertexbuffer create failure!"); return; }

    DXVertex *dxPrimitiveCubeVertices = NULL;
    dxPrimitiveCubeVertexBuffer->Lock(0, 0, (void**)&dxPrimitiveCubeVertices, 0);
    if(!dxPrimitiveCubeVertices) { wError("dxPrimitiveInit vertexbuffer lock failure!"); return; }

    d3dDevice->CreateIndexBuffer( dxPrimitiveCubeNumIndices * sizeof(short), 0, D3DFMT_INDEX16, D3DPOOL_MANAGED, &dxPrimitiveCubeIndexBuffer, NULL);
    if(!dxPrimitiveCubeIndexBuffer) { wError("dxPrimitiveInit indexbuffer create failure!"); return; }

    short *cubeIndices = NULL;
    dxPrimitiveCubeIndexBuffer->Lock( 0, 0, (void**)&cubeIndices, 0);
    if(!cubeIndices) { wError("dxPrimitiveInit indexbuffer lock failure!"); return; }
    memcpy(cubeIndices, dxPrimitiveCubeIndices, sizeof(dxPrimitiveCubeIndices));
    for(int a = 0; a < 24; a++) dxPrimitiveCubeVertices[a].reset();
    // --> Front Face
    for(int a = 0; a <= 3; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
    dxPrimitiveCubeVertices[0].setAllUV(0.0f, 0.0f); dxPrimitiveCubeVertices[0].pos = D3DXVECTOR3(-1.0f, -1.0f, 1.0f); // Point 1 (Front)
    dxPrimitiveCubeVertices[1].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[1].pos = D3DXVECTOR3( 1.0f, -1.0f, 1.0f); // Point 2 (Front)
    dxPrimitiveCubeVertices[2].setAllUV(0.0f, 1.0f); dxPrimitiveCubeVertices[2].pos = D3DXVECTOR3(-1.0f,  1.0f, 1.0f); // Point 3 (Front)
    dxPrimitiveCubeVertices[3].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[3].pos = D3DXVECTOR3( 1.0f,  1.0f, 1.0f); // Point 4 (Front)
    // --> Back Face
    for(int a = 4; a <= 7; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(0.0f, 0.0f,-1.0f);
    dxPrimitiveCubeVertices[4].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[4].pos = D3DXVECTOR3(-1.0f, -1.0f,-1.0f); // Point 1 (Back)
    dxPrimitiveCubeVertices[5].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[5].pos = D3DXVECTOR3(-1.0f,  1.0f,-1.0f); // Point 2 (Back)
    dxPrimitiveCubeVertices[6].setAllUV(0.0f, 0.0f); dxPrimitiveCubeVertices[6].pos = D3DXVECTOR3( 1.0f, -1.0f,-1.0f); // Point 3 (Back)
    dxPrimitiveCubeVertices[7].setAllUV(0.0f, 1.0f); dxPrimitiveCubeVertices[7].pos = D3DXVECTOR3( 1.0f,  1.0f,-1.0f); // Point 4 (Back)
    // --> Top Face
    for(int a = 8; a <= 11; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(0.0f, 1.0f, 0.0f);
    dxPrimitiveCubeVertices[8].setAllUV(0.0f, 1.0f);  dxPrimitiveCubeVertices[8].pos  = D3DXVECTOR3(-1.0f,  1.0f,-1.0f); // Point 1 (Top)
    dxPrimitiveCubeVertices[9].setAllUV(0.0f, 0.0f);  dxPrimitiveCubeVertices[9].pos  = D3DXVECTOR3(-1.0f,  1.0f, 1.0f); // Point 2 (Top)
    dxPrimitiveCubeVertices[10].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[10].pos = D3DXVECTOR3( 1.0f,  1.0f,-1.0f); // Point 3 (Top)
    dxPrimitiveCubeVertices[11].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[11].pos = D3DXVECTOR3( 1.0f,  1.0f, 1.0f); // Point 4 (Top)
    // --> Bottom Face
    for(int a = 12; a <= 15; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(0.0f,-1.0f, 0.0f);
    dxPrimitiveCubeVertices[12].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[12].pos = D3DXVECTOR3(-1.0f, -1.0f,-1.0f); // Point 1 (Bottom)
    dxPrimitiveCubeVertices[13].setAllUV(0.0f, 1.0f); dxPrimitiveCubeVertices[13].pos = D3DXVECTOR3( 1.0f, -1.0f,-1.0f); // Point 2 (Bottom)
    dxPrimitiveCubeVertices[14].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[14].pos = D3DXVECTOR3(-1.0f, -1.0f, 1.0f); // Point 3 (Bottom)
    dxPrimitiveCubeVertices[15].setAllUV(0.0f, 0.0f); dxPrimitiveCubeVertices[15].pos = D3DXVECTOR3( 1.0f, -1.0f, 1.0f); // Point 4 (Bottom)
    // --> Right Face
    for(int a = 16; a <= 19; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(1.0f, 0.0f, 0.0f);
    dxPrimitiveCubeVertices[16].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[16].pos = D3DXVECTOR3( 1.0f, -1.0f,-1.0f); // Point 1 (Right)
    dxPrimitiveCubeVertices[17].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[17].pos = D3DXVECTOR3( 1.0f,  1.0f,-1.0f); // Point 2 (Right)
    dxPrimitiveCubeVertices[18].setAllUV(0.0f, 0.0f); dxPrimitiveCubeVertices[18].pos = D3DXVECTOR3( 1.0f, -1.0f, 1.0f); // Point 3 (Right)
    dxPrimitiveCubeVertices[19].setAllUV(0.0f, 1.0f); dxPrimitiveCubeVertices[19].pos = D3DXVECTOR3( 1.0f,  1.0f, 1.0f); // Point 4 (Right)
    // --> Left Face
    for(int a = 20; a <= 23; a++) dxPrimitiveCubeVertices[a].normal = D3DXVECTOR3(-1.0f, 0.0f, 0.0f);
    dxPrimitiveCubeVertices[20].setAllUV(0.0f, 0.0f); dxPrimitiveCubeVertices[20].pos = D3DXVECTOR3(-1.0f, -1.0f,-1.0f); // Point 1 (Left)
    dxPrimitiveCubeVertices[21].setAllUV(1.0f, 0.0f); dxPrimitiveCubeVertices[21].pos = D3DXVECTOR3(-1.0f, -1.0f, 1.0f); // Point 2 (Left)
    dxPrimitiveCubeVertices[22].setAllUV(0.0f, 1.0f); dxPrimitiveCubeVertices[22].pos = D3DXVECTOR3(-1.0f,  1.0f,-1.0f); // Point 3 (Left)
    dxPrimitiveCubeVertices[23].setAllUV(1.0f, 1.0f); dxPrimitiveCubeVertices[23].pos = D3DXVECTOR3(-1.0f,  1.0f, 1.0f); // Point 4 (Left)
    // --> Unlock cube
    dxPrimitiveCubeVertexBuffer->Unlock();
    dxPrimitiveCubeIndexBuffer->Unlock();
}

//! Releases the global primitive vertexbuffers
void dxPrimitiveRelease() {
    if(dxPrimitiveCubeVertexBuffer) dxPrimitiveCubeVertexBuffer->Release();
    if(dxPrimitiveCubeIndexBuffer) dxPrimitiveCubeIndexBuffer->Release();
    if(dxTeapotMesh) dxTeapotMesh->Release();
}

//! Renders a basic unit cube
void dxCube() {
    d3dDevice->SetFVF(DXVERTEXFVF);
    d3dDevice->SetStreamSource(0, dxPrimitiveCubeVertexBuffer, 0, sizeof(DXVertex));
    d3dDevice->SetIndices(dxPrimitiveCubeIndexBuffer);
    d3dDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, 24, 0, 12); 
}

//! Renders a minimal unit quad
void dxFastQuad() {
    if( !dxPrimitiveFastQuadVertexBuffer ) {
        DXVertex4 *qVertex = NULL;
        d3dDevice->CreateVertexBuffer( 4 * sizeof(DXVertex4), 0, DXVERTEXFVF4, D3DPOOL_MANAGED, &dxPrimitiveFastQuadVertexBuffer, NULL);
        dxPrimitiveFastQuadVertexBuffer->Lock(0, 0, (void**)&qVertex, 0);
        qVertex[0].pos = D3DXVECTOR3(-1.0f,-1.0f,0.0f); qVertex[0].setAllUV( 0.0f, 0.0f);
        qVertex[1].pos = D3DXVECTOR3( 1.0f,-1.0f,0.0f); qVertex[1].setAllUV( 1.0f, 0.0f);
        qVertex[2].pos = D3DXVECTOR3(-1.0f, 1.0f,0.0f); qVertex[2].setAllUV( 0.0f, 1.0f);
        qVertex[3].pos = D3DXVECTOR3( 1.0f, 1.0f,0.0f); qVertex[3].setAllUV( 1.0f, 1.0f);
        dxPrimitiveFastQuadVertexBuffer->Unlock();
    }
    d3dDevice->SetFVF(DXVERTEXFVF4);
    d3dDevice->SetStreamSource(0, dxPrimitiveFastQuadVertexBuffer, 0, sizeof(DXVertex4));
    d3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
}

//! Renders a quad
void dxQuad( float width, float height, float xo, float yo, DWORD colour, bool RHW ) {
    LPDIRECT3DVERTEXBUFFER9 bgVertexBuffer = NULL; DXVertex2 *bgVertex = NULL;
    size_t vertexSz = RHW?sizeof(DXVertex2):sizeof(DXVertex);
    d3dDevice->CreateVertexBuffer( 4 * vertexSz, 0, RHW?DXVERTEXFVF2:DXVERTEXFVF, D3DPOOL_MANAGED, &bgVertexBuffer, NULL);
    bgVertexBuffer->Lock(0, 0, (void**)&bgVertex, 0);
    for(int a = 0; a < 4; a++) { bgVertex[a].diffuse = 0xffffffff; bgVertex[a].rhw = 1.0f; }
    if( RHW ) {
        bgVertex[0].pos = D3DXVECTOR3(xo      , yo       ,1.0f); bgVertex[0].u = 0.0f; bgVertex[0].v = 0.0f;
        bgVertex[1].pos = D3DXVECTOR3(width+xo, yo       ,1.0f); bgVertex[1].u = 1.0f; bgVertex[1].v = 0.0f;
        bgVertex[2].pos = D3DXVECTOR3(xo      , height+yo,1.0f); bgVertex[2].u = 0.0f; bgVertex[2].v = 1.0f;
        bgVertex[3].pos = D3DXVECTOR3(width+xo, height+yo,1.0f); bgVertex[3].u = 1.0f; bgVertex[3].v = 1.0f;
    } else {
        DXVertex *qVertex = (DXVertex*) bgVertex;
        qVertex[0].reset(); qVertex[1].reset(); qVertex[2].reset(); qVertex[3].reset();
        qVertex[0].pos = D3DXVECTOR3(xo      , yo       ,0.0f); qVertex[0].setAllUV( 0.0f, 0.0f);
        qVertex[1].pos = D3DXVECTOR3(width+xo, yo       ,0.0f); qVertex[1].setAllUV( 1.0f, 0.0f);
        qVertex[2].pos = D3DXVECTOR3(xo      , height+yo,0.0f); qVertex[2].setAllUV( 0.0f, 1.0f);
        qVertex[3].pos = D3DXVECTOR3(width+xo, height+yo,0.0f); qVertex[3].setAllUV( 1.0f, 1.0f);
    }
    bgVertexBuffer->Unlock();
    d3dDevice->SetFVF(RHW?DXVERTEXFVF2:DXVERTEXFVF);
    d3dDevice->SetStreamSource(0, bgVertexBuffer, 0, vertexSz);
    d3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
    bgVertexBuffer->Release();
}

//! Renders a fullscreen picture
void dxBackgroundPicture(int width, int height, DWORD colour) {
    dxQuad( width, height, -0.5f, -0.5f, colour, true );
}

//! Renders a teapot
void dxTeapot() {
    d3dDevice->SetFVF(DXVERTEXFVF);
    dxTeapotMesh->DrawSubset(0);
}

//! Calculates triangle tangent vector, given 3 UV-assigned vertexes
void dxPrimitiveTangent( DXVertex3 &v1, DXVertex3 &v2, DXVertex3 &v3) {
    float x1 = v2.pos.x - v1.pos.x;
    float x2 = v3.pos.x - v1.pos.x;
    float y1 = v2.pos.y - v1.pos.y;
    float y2 = v3.pos.y - v1.pos.y;
    float z1 = v2.pos.z - v1.pos.z;
    float z2 = v3.pos.z - v1.pos.z;
    float s1 = v2.uv.x - v1.uv.x;
    float s2 = v3.uv.x - v1.uv.x;
    float t1 = v2.uv.y - v1.uv.y;
    float t2 = v3.uv.y - v1.uv.y;
    float r = 1.0f / (s1 * t2 - s2 * t1);
    D3DXVECTOR3 sdir((t2 * x1 - t1 * x2) * r, (t2 * y1 - t1 * y2) * r, (t2 * z1 - t1 * z2) * r);
    D3DXVECTOR3 tdir((s1 * x2 - s2 * x1) * r, (s1 * y2 - s2 * y1) * r, (s1 * z2 - s2 * z1) * r);
    v1.tangent = ( sdir - (v1.normal * D3DXVec3Dot(&v1.normal, &sdir)) );
    v2.tangent = ( sdir - (v1.normal * D3DXVec3Dot(&v2.normal, &sdir)) );
    v3.tangent = ( sdir - (v1.normal * D3DXVec3Dot(&v3.normal, &sdir)) );
    D3DXVec3Normalize( &v1.tangent, &v1.tangent );
    D3DXVec3Normalize( &v2.tangent, &v2.tangent );
    D3DXVec3Normalize( &v3.tangent, &v3.tangent );
    D3DXVec3Cross( &v1.binormal, &v1.normal, &v1.tangent );
    D3DXVec3Cross( &v2.binormal, &v2.normal, &v2.tangent );
    D3DXVec3Cross( &v3.binormal, &v3.normal, &v3.tangent );
}

//! Calculates vertex on a unit sphere, given two angles
D3DXVECTOR3 dxPrimitiveCreateSpherePoint( float angleStack, float angleSlice ) {
    return D3DXVECTOR3( 
        sin( angleStack ) * cos( angleSlice ),
        cos( angleStack ),
        sin( angleStack ) * sin( angleSlice )
    );
}

//! Creates a sphere primitive object
dxPrimitivesObj *dxPrimitiveCreateSphereMesh(float radius, unsigned int slices, unsigned int stacks) {
    if( slices == 0 || stacks == 0 ) return NULL;

    dxPrimitivesObj *dxSphere = new dxPrimitivesObj;

    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );

    d3dDevice->CreateVertexBuffer(slices * stacks * 6 * sizeof(DXVertex3), 0, 0, D3DPOOL_MANAGED, &dxSphere->vertexBuffer, NULL);
    if(!dxSphere->vertexBuffer) { wError("dxPrimitiveCreateSphereMesh vertexbuffer create failure!"); return NULL; }

    DXVertex3 *dxSphereVertices = NULL;
    dxSphere->vertexBuffer->Lock(0, 0, (void**)&dxSphereVertices, 0);
    if(!dxSphereVertices) { wError("dxPrimitiveCreateSphereMesh vertexbuffer lock failure!"); return NULL; }

    float sliceStep = (2.0f * D3DX_PI) / (float)slices;
    float stackStep = D3DX_PI / (float)stacks;

    dxSphere->nPolygons = 0;

    for(int a = 0, i = 0; a < stacks; a++) {
        float curStack = ( (float)a * stackStep );
        float nxtStack = ( (float)(a + 1) * stackStep );
        for(int b = 0; b < slices; b++) {
            float curSlice = ( (float)b * sliceStep );
            float nxtSlice = ( (float)(b + 1) * sliceStep );

            D3DXVECTOR3 pos0 = radius * dxPrimitiveCreateSpherePoint( curStack, curSlice );
            D3DXVECTOR3 pos1 = radius * dxPrimitiveCreateSpherePoint( curStack, nxtSlice );
            D3DXVECTOR3 pos2 = radius * dxPrimitiveCreateSpherePoint( nxtStack, curSlice );
            D3DXVECTOR3 pos3 = radius * dxPrimitiveCreateSpherePoint( nxtStack, nxtSlice );

            // First Polygon
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos0;
            dxSphereVertices[i].setAllUV( ((float)(b))/((float)(slices)), ((float)(a))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos1;
            dxSphereVertices[i].setAllUV( ((float)(b+1))/((float)(slices)), ((float)(a))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos2;
            dxSphereVertices[i].setAllUV( ((float)(b))/((float)(slices)), ((float)(a+1))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxPrimitiveTangent( dxSphereVertices[i-3], dxSphereVertices[i-2], dxSphereVertices[i-1] );
            dxSphere->nPolygons++;

            // Second Polygon
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos2;
            dxSphereVertices[i].setAllUV( ((float)(b))/((float)(slices)), ((float)(a+1))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos1;
            dxSphereVertices[i].setAllUV( ((float)(b+1))/((float)(slices)), ((float)(a))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxSphereVertices[i].reset();
            dxSphereVertices[i].pos = pos3;
            dxSphereVertices[i].setAllUV(  ((float)(b+1))/((float)(slices)), ((float)(a+1))/((float)(stacks)) );
            D3DXVec3Normalize(&dxSphereVertices[i].normal, &dxSphereVertices[i].pos);
            i++;
            dxPrimitiveTangent( dxSphereVertices[i-3], dxSphereVertices[i-2], dxSphereVertices[i-1] );
            dxSphere->nPolygons++;

        }
    }

    dxSphere->vertexBuffer->Unlock();
    dxSphere->slices = slices;
    dxSphere->stacks = stacks;

    return dxSphere;
}

//! Renders a generic primitive object
void dxPrimitiveModel( dxPrimitivesObj *modelObj ) {
    if(!modelObj) return;
    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );
    d3dDevice->SetStreamSource(0, modelObj->vertexBuffer, 0, sizeof(DXVertex3));
    d3dDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, modelObj->nPolygons);
}

//! Renders a sphere primitive object
void dxSphere( dxPrimitivesObj *meshSphere ) { dxPrimitiveModel( meshSphere ); }

//! Creates a ring primitive object
dxPrimitivesObj *dxPrimitiveCreateRing(float radius, float innerRadius, int slices) {
    dxPrimitivesObj *dxRing = new dxPrimitivesObj;

    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );

    d3dDevice->CreateVertexBuffer((slices+1) * 2 * sizeof(DXVertex3), 0, 0, D3DPOOL_MANAGED, &dxRing->vertexBuffer, NULL);
    if(!dxRing->vertexBuffer) { wError("dxRing vertexbuffer create failure!"); return NULL; }

    DXVertex3 *dxRingVertices = NULL;
    dxRing->vertexBuffer->Lock(0, 0, (void**)&dxRingVertices, 0);
    if(!dxRingVertices) { wError("dxRing vertexbuffer lock failure!"); return NULL; }

    float stepAngle = 360.0f / ((float)(slices));
    float curAngle = 0.0f;
    float x, y;
    for(int a = 0; a <= slices; a++, curAngle += stepAngle) {
        x = sinf( D3DXToRadian(curAngle) ); y = cosf( D3DXToRadian(curAngle) );
        int i = a*2;
        dxRingVertices[i].reset();
        dxRingVertices[i].pos = D3DXVECTOR3( x * radius, y * radius, 0.0f );
        dxRingVertices[i].normal = D3DXVECTOR3( 0.0, 0.0, -1.0f );
        dxRingVertices[i].setAllUV( 1.0f, ((float)(a))/((float)(slices)) );
        i++;
        dxRingVertices[i].reset();
        dxRingVertices[i].pos = D3DXVECTOR3( x * innerRadius, y * innerRadius, 0.0f );
        dxRingVertices[i].normal = D3DXVECTOR3( 0.0, 0.0, -1.0f );
        dxRingVertices[i].setAllUV( 0.0f, ((float)(a))/((float)(slices)) );
    }

    dxRing->vertexBuffer->Unlock();

    dxRing->slices = slices;

    return dxRing;
}

//! Renders a ring primitive object
void dxRing( dxPrimitivesObj *objRing ) {
    if(!objRing) return;
    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );
    d3dDevice->SetStreamSource(0, objRing->vertexBuffer, 0, sizeof(DXVertex3));
    d3dDevice->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2 * objRing->slices);
}

//! Creates a tube primitive object
dxPrimitivesObj *dxPrimitiveCreateTube(float radius, float length, int slices, int sections) {
    dxPrimitivesObj *dxTube = new dxPrimitivesObj;

    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );
    d3dDevice->CreateVertexBuffer(6 * slices * sections * sizeof(DXVertex3), 0, 0, D3DPOOL_MANAGED, &dxTube->vertexBuffer, NULL);
    if(!dxTube->vertexBuffer) { wError("dxPrimitiveCreateTube vertexbuffer create failure!"); return NULL; }

    DXVertex3 *dxTubeVertices = NULL;
    dxTube->vertexBuffer->Lock(0, 0, (void**)&dxTubeVertices, 0);
    if(!dxTubeVertices) { wError("dxPrimitiveCreateTube vertexbuffer lock failure!"); return NULL; }

    float stepAngle = 360.0f / ((float)(slices));
    float curAngle = 0.0f;
    float stepHeight = length / ((float)(sections));
    float heightOffset = length * 0.5;
    float x0, y0, x1, y1;
    int i = 0;
    for(int a = 0; a < slices; a++, curAngle += stepAngle) {
        x0 = sinf( D3DXToRadian(curAngle) ); y0 = cosf( D3DXToRadian(curAngle) );
        x1 = sinf( D3DXToRadian(curAngle + stepAngle) ); y1 = cosf( D3DXToRadian(curAngle + stepAngle) );
        for(int b = 0; b < sections; b++) {
            float z0 = b * stepHeight - heightOffset, z1 = (b + 1) * stepHeight - heightOffset;

            dxTubeVertices[i].reset();
            dxTubeVertices[i].pos = D3DXVECTOR3( x0 * radius, y0 * radius, z0 );
            dxTubeVertices[i].normal = D3DXVECTOR3( x0, y0, 0.0f );
            dxTubeVertices[i].setAllUV( ((float)(a))/((float)(slices)), ((float)(b))/((float)(sections)) );
            i++;

            dxTubeVertices[i].reset();
            dxTubeVertices[i].pos = D3DXVECTOR3( x0 * radius, y0 * radius, z1 );
            dxTubeVertices[i].normal = D3DXVECTOR3( x0, y0, 0.0f );
            dxTubeVertices[i].setAllUV( ((float)(a))/((float)(slices)), ((float)(b+1))/((float)(sections)) );
            i++;

            dxTubeVertices[i].reset();
            dxTubeVertices[i].pos = D3DXVECTOR3( x1 * radius, y1 * radius, z0 );
            dxTubeVertices[i].normal = D3DXVECTOR3( x1, y1, 0.0f );
            dxTubeVertices[i].setAllUV( ((float)(a+1))/((float)(slices)), ((float)(b))/((float)(sections)) );
            i++;

            dxTubeVertices[i].reset(); dxTubeVertices[i] = dxTubeVertices[i-1]; i++;
            dxTubeVertices[i].reset(); dxTubeVertices[i] = dxTubeVertices[i-3]; i++;

            dxTubeVertices[i].reset();
            dxTubeVertices[i].pos = D3DXVECTOR3( x1 * radius, y1 * radius, z1 );
            dxTubeVertices[i].normal = D3DXVECTOR3( x1, y1, 0.0f );
            dxTubeVertices[i].setAllUV( ((float)(a+1))/((float)(slices)), ((float)(b+1))/((float)(sections)) );
            i++;

        }
    }
    dxTube->vertexBuffer->Unlock();
    dxTube->nPolygons = 2 * slices * sections;
    dxTube->slices = slices;
    dxTube->stacks = sections;
    return dxTube;
}

//! Calculates a point on a torus
D3DXVECTOR3 dxPrimitiveCreateTorusPoint( float radius, float radius2, float curAngle, float curAngle2 ) {     
    return D3DXVECTOR3 (
        (radius + radius2 * cosf( D3DXToRadian( curAngle ) ) ) * cosf( D3DXToRadian( curAngle2 ) ),
        (radius + radius2 * cosf( D3DXToRadian( curAngle ) ) ) * sinf( D3DXToRadian( curAngle2 ) ),
        radius2 * sinf( D3DXToRadian( curAngle ) )
    );
}

//! Calculates the ring center co-ordinates of a torus
D3DXVECTOR3 dxPrimitiveCreateTorusCenter( float radius, float curAngle2 ) {
    return D3DXVECTOR3 (
        radius * cosf( D3DXToRadian( curAngle2 ) ),
        radius * sinf( D3DXToRadian( curAngle2 ) ),
        0.0f
    );
}

//! Creates a torus primitive object
dxPrimitivesObj *dxPrimitiveCreateTorus(float radius, float radius2, int slices, int sections) {
    dxPrimitivesObj *dxTorus = new dxPrimitivesObj;

    d3dDevice->SetVertexDeclaration( _DXVERTEXDECL3 );
    d3dDevice->CreateVertexBuffer(6 * slices * sections * sizeof(DXVertex3), 0, 0, D3DPOOL_MANAGED, &dxTorus->vertexBuffer, NULL);
    if(!dxTorus->vertexBuffer) { wError("dxPrimitiveCreateTorus vertexbuffer create failure!"); return NULL; }

    DXVertex3 *dxTorusVertices = NULL;
    dxTorus->vertexBuffer->Lock(0, 0, (void**)&dxTorusVertices, 0);
    if(!dxTorusVertices) { wError("dxPrimitiveCreateTorus vertexbuffer lock failure!"); return NULL; }

    D3DXVECTOR3 c0(0.0f, 0.0f, 0.0f), c1(0.0f, 0.0f, 0.0f), c2(0.0f, 0.0f, 0.0f), c3(0.0f, 0.0f, 0.0f);
    D3DXVECTOR3 center0, center1;

    float stepAngle = 360.0f / ((float)(slices));
    float stepAngle2 = 360.0f / ((float)(sections));
    float curAngle = 0.0f, curAngle2 = 0.0f;

    int i = 0;
    for(int a = 0; a < slices; a++, curAngle += stepAngle) {
        curAngle2 = 0.0f;
        for(int b = 0; b < sections; b++, curAngle2 += stepAngle2) {

            c0 = dxPrimitiveCreateTorusPoint( radius, radius2, curAngle, curAngle2 );
            c1 = dxPrimitiveCreateTorusPoint( radius, radius2, curAngle, curAngle2 + stepAngle2 );
            c2 = dxPrimitiveCreateTorusPoint( radius, radius2, curAngle + stepAngle, curAngle2 );
            c3 = dxPrimitiveCreateTorusPoint( radius, radius2, curAngle + stepAngle, curAngle2 + stepAngle2 );
            center0 = dxPrimitiveCreateTorusCenter( radius, curAngle2 );
            center1 = dxPrimitiveCreateTorusCenter( radius, curAngle2 + stepAngle2 );

            dxTorusVertices[i].reset();
            dxTorusVertices[i].pos = c0;
            dxTorusVertices[i].normal = c0 - center0;
            dxTorusVertices[i].setAllUV( ((float)(a))/((float)(slices)), ((float)(b))/((float)(sections)) );
            i++;

            dxTorusVertices[i].reset();
            dxTorusVertices[i].pos = c1;
            dxTorusVertices[i].normal = c1 - center1;
            dxTorusVertices[i].setAllUV( ((float)(a))/((float)(slices)), ((float)(b+1))/((float)(sections)) );
            i++;

            dxTorusVertices[i].reset();
            dxTorusVertices[i].pos = c2;
            dxTorusVertices[i].normal = c2  - center0;
            dxTorusVertices[i].setAllUV( ((float)(a+1))/((float)(slices)), ((float)(b))/((float)(sections)) );
            i++;

            dxTorusVertices[i].reset(); dxTorusVertices[i] = dxTorusVertices[i-1]; i++;
            dxTorusVertices[i].reset(); dxTorusVertices[i] = dxTorusVertices[i-3]; i++;

            dxTorusVertices[i].reset();
            dxTorusVertices[i].pos = c3;
            dxTorusVertices[i].normal = c3  - center1;
            dxTorusVertices[i].setAllUV( ((float)(a+1))/((float)(slices)), ((float)(b+1))/((float)(sections)) );
            i++;

        }
    }

    dxTorus->vertexBuffer->Unlock();
    dxTorus->nPolygons = 2 * slices * sections;
    dxTorus->slices = slices;
    dxTorus->stacks = sections;
    
    return dxTorus;
}

//! Renders a tube primitive object
void dxTube( dxPrimitivesObj *objTube ) { dxPrimitiveModel( objTube ); }

//! Renders a torus primitive object
void dxTorus( dxPrimitivesObj *objTorus ) { dxPrimitiveModel( objTorus ); }

//! Extrudes given polygon edge and puts result into givern polygon queue
void dxExtrude( DXVertex3 v0, DXVertex3 v1, float dis, PolygonQueue &pq ) {
    D3DXVECTOR3 vUp( 0.0, 0.0, 1.0 );
    D3DXVECTOR3 vNormal;
    DXVertex3 p00 = v0, p01 = v1, p10 = v0, p11 = v1;
    p10.pos.z += dis; p11.pos.z += dis;
    D3DXVec3Cross(&vNormal, &(v1.pos - v0.pos), &vUp);
    p00.normal = p01.normal = p10.normal = p11.normal = vNormal;
    // First triangle
    dxPrimitiveTangent( p00, p01, p10 );
    pq.pushVertex( &p00 ); pq.pushVertex( &p01 );pq.pushVertex( &p10 );
    // Second triangle
    dxPrimitiveTangent( p10, p01, p11 );
    pq.pushVertex( &p10 ); pq.pushVertex( &p01 );pq.pushVertex( &p11 );
}
