/** \file DXTexture.cpp
<h2> Direct3D Texture Manager </h2> <br>
Copyright (C) 2011 Xi Chen <br>

Supports BMP, JPG, TGA, PNG, DDS, PPM, DIB, HDR, PFM

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Texture Manager
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../EngineHeader/DXTexture.h"

// --------> Texture Methods

Texture::Texture() { image = NULL; }

Texture::~Texture() { }

/**
Loads a texture given fileName
\param fileName Relative file path to image
\param filter Texture filter 
*/
void Texture::load(const char *fileName, char filter) {
    release();
    HRESULT ret = D3DXCreateTextureFromFile(d3dDevice, fileName, &image);
    switch( ret ) {
        case D3D_OK: return;
        case D3DERR_NOTAVAILABLE: wError("Texture::Load() Error - D3DERR_NOTAVAILABLE"); return;
        case D3DERR_OUTOFVIDEOMEMORY: wError("Texture::Load() Error - D3DERR_OUTOFVIDEOMEMORY"); return;
        case D3DERR_INVALIDCALL: wError("Texture::Load() Error - D3DERR_INVALIDCALL"); return;
        case D3DXERR_INVALIDDATA: wError("Texture::Load() Error - D3DXERR_INVALIDDATA"); return;
        case E_OUTOFMEMORY: wError("Texture::Load() Error - E_OUTOFMEMORY"); return;
    }
}

//! Release this texture
void Texture::release() {
    if(image) image->Release();
    image = NULL;
}

//! Returns the Direct3D texture handle
Texture::operator LPDIRECT3DTEXTURE9() {
    return image;
}

// --------> TextureArray Methods

//! Frees up all texture and image data
void TextureArray::clear() {
    for(int a = 0; a < textureData.size(); a++) textureData[a].release();
    textureData.clear();
}

/**
Loads an image and builds texture
Resizes the vector to hold elements 0 -> texID.
\param texID Texture ID number
\param fileName Relative file path to image (PNG or 24-bit BMP formats)
\param filter (optional)1 = Mipmapped, 0 = Linear filtered
\warning Don't put huge numbers for texID, as it will make an array big enough to hold that index.
*/
void TextureArray::load(int texID, const char *fileName, char filter) {
    while (texID >= textureData.size()) {
        Texture emptyTexture;
        textureData.push_back(emptyTexture);
    }
    textureData[texID].load(fileName, filter);
}

//! The [] operator, so you get go d3dDevice->SetTexture(0, TextureArray[0]);
LPDIRECT3DTEXTURE9 TextureArray::operator [] (int texID) {
    if(texID >= textureData.size()) {wError("TextureArray::[] Bad texID");return 0;}
    return textureData[texID].image;
}

TextureArray::~TextureArray() { this->clear(); }



