/** \file SceneAbstract.h
<h2> Abstract Scenes </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Abstract Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_ABSTRACT_H__
#define __SCENE_ABSTRACT_H__

#include "Scene.h"

#define SCENE_ABS_BEZIERSTEPS 8
#define SCENE_ABS_CURVES 32
#define SCENE_ABS_NUMBEAMS 8

struct SceneAbstractLineSegment {
    D3DXVECTOR3 p1;
    D3DXVECTOR3 d1;
    D3DXVECTOR3 p2;
    D3DXVECTOR3 d2;
    float z;
    inline bool operator < (SceneAbstractLineSegment &a) {
        return z > a.z;
    }
};

class SceneAbstract: public Scene {
protected:
    TextureArray textures;

    // --> Curves scene
    Shader lineShader;
    vector<D3DXVECTOR3> bzPoints[SCENE_ABS_CURVES];
    bool ZSort;

    float bezierFunction( int i, double t );
    float bezierFunctionDerivative( int i, double t );
    D3DXVECTOR3 evaluateBezier( int i, double t, vector<D3DXVECTOR3> &pts );
    D3DXVECTOR3 evaluateBezierDerivative( int i, double t, vector<D3DXVECTOR3> &pts );

    void renderCurves(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Fill scene
    Shader fillShader;

    void renderFill(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Ion scene
    double ionTimeScale;
    Model ionModel;
    Model ionConduitModel;
    Shader ionShader;
    dxPrimitivesObj *ionBeam;

    void renderIon(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> City scene
    Model cityModel;
    Model cityStreetModel;
    Shader cityShader;
    LPDIRECT3DCUBETEXTURE9  cityCubeMap;
    dxPrimitivesObj *citySkyDome;

    void renderCity(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Lucy scene
    Model lucyModel;
    Shader lucyShader;
    LPDIRECT3DCUBETEXTURE9  lucyIrradianceMap;
    LPDIRECT3DCUBETEXTURE9  lucyReflectionMap;

    void renderLucy(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

    // --> Aneurism scene
    Shader aneurismShader;
    dxPrimitivesObj *aneurismSphere;
    LPDIRECT3DVOLUMETEXTURE9 aneurismVolume;
    void renderAneurism(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);

public:
    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();
};



#endif