/** \file SceneHuman.h
<h2> NVIDIA Human Head Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief NVIDIA Human Head Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_HUMAN_HEAD_H__
#define __SCENE_HUMAN_HEAD_H__

#include "Scene.h"
#include "../EngineHeader/DXShadow.h"

#define SCENEHUMAN_DEFAULT_IRR_TEXSIZE 1024
#define SCENEHUMAN_NUM_GAUSSIANS 6
#define SCENEHUMAN_DEFAULT_DIFFUSE_VARIANCE_SCALE 25.0f
#define SCENEHUMAN_DEFAULT_SHADOWMAP_SIZE 1024
#define SCENEHUMAN_SHADOWMAP_BOXSIZE 200

class SceneHuman : public Scene {

    TextureArray textures;
    dxPrimitivesObj *humanSkyDome;
    Model humanHeadModel;
    bool displayImageDumps;

    Shader irradianceShader;
    RenderPass irradianceTexturePass;
    int irradianceTextureSize;

    Shader diffuseGaussianShader;
    RenderPass diffusePassU[ SCENEHUMAN_NUM_GAUSSIANS ];
    RenderPass diffusePassV[ SCENEHUMAN_NUM_GAUSSIANS ];
    float diffuseVarianceModel[ SCENEHUMAN_NUM_GAUSSIANS ];
    float diffuseVarianceScale;

    Shader skinShader;
    LPDIRECT3DCUBETEXTURE9 skinCubeIrradianceMap;

    ShadowMap headShadowMap;
    Shader headShadowMapShader;
    int headShadowMapSize;

    D3DXVECTOR3 headLightPos;
    float headAmbientAmount;

public:
    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();
};

#endif