/** \file SceneTerrain.h
<h2> Elevated (Terrain) Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Elevated (Terrain) Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#ifndef __SCENE_TERRAIN_H__
#define __SCENE_TERRAIN_H__

#include "Scene.h"

#define SCENETERRAIN_SHADER_FILE "Terrain/proceduralTerrainShader.fx"
#define SCENETERRAIN_SKYSHADER_FILE "Terrain/skyShader.fx"
#define SCENETERRAIN_TEX0 "Terrain/rock.jpg"
#define SCENETERRAIN_TEX1 "Terrain/snow.jpg"
#define SCENETERRAIN_TEX2 "Terrain/noise.bmp"
#define SCENETERRAIN_TEXNAME0 "rockTexture"
#define SCENETERRAIN_TEXNAME1 "snowTexture"
#define SCENETERRAIN_TEXNAME2 "noiseTexture"
#define SCENETERRAIN_SKYDOME_RADIUS 2500.0f
#define SCENETERRAIN_MOVESCALE 0.01f

class SceneTerrain: public Scene {
    // Skydome
    Shader skyShader;
    // Terrain shader
    Shader terrainShader;
    TextureArray terrainTexture;
    // Terrain mesh parameters
    int terrainRes;
    float terrainSize;
    // Terrain mesh stats
    int nVertices;
    int nPolygons;
    // Terrain mesh
    LPDIRECT3DVERTEXBUFFER9 vertexBuffer;
    LPDIRECT3DINDEXBUFFER9 indexBuffer;
    // Terran light position
    D3DXVECTOR3 terrainSunPos;
    D3DXVECTOR3 terranAmbientColour1;
    D3DXVECTOR3 terranAmbientColour2;
    D3DXVECTOR3 terranFogColour1;
    D3DXVECTOR3 terranFogColour2;
public:
    float terrainMoveX;
    float terrainMoveY;

    void load();
    void render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection);
    void update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem );
    void release();

    void addParamVector( ParamPath *dest );
    void updateParamVector( ParamPath &paramPath, double pathTime );
};


#endif
