/** \file SceneCar.cpp
<h2> Car Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Car Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../GameHeader/SceneCar.h"

int dxSceneCarMaterialFunction( int n ) {
    switch ( n ) {
        case 8: return 5;
        case 9 : case 15: case 18: case 21: return 9;
        case 10: case 16: case 19: case 22: return 10;
        case 11: case 17: case 20: case 23: return 11;
        case 12: case 13: return 0;
    }
    return n;
}

// --------> SceneCar Methods

void SceneCar::load() {
    loadSceneAnimationPaths( "Car/anim.path" );
    carModel.materialsEnabled = true;
    carModel.materialDebug = -1;
    carModel.materialCallback = dxSceneCarMaterialFunction;
    carModel.load("Car/car.3ds");
    carShader.loadFile("Car/carShader.fx");
    carTextures.load( 0, "Car/lights.tga" );
    carTextures.load( 1, "Car/interior.tga" );
    carTextures.load( 2, "Car/black.tga" );
    carTextures.load( 3, "Car/under.tga" );
    carTextures.load( 4, "Car/grille2.tga" );
    carTextures.load( 5, "Car/tire.tga" );
    carTextures.load( 6, "Car/rim.tga" );
    carTextures.load( 7, "Car/rim2.tga" );
    carTextures.load( 8, "Car/bdrf.dds" );
    carTextures.load( 9, "Car/tireNormal.tga" );
    carTextures.load(10, "Car/interiorNormal.tga" );
    D3DXCreateCubeTextureFromFile( d3dDevice, "Car/env.dds", &carCubeMap );
    D3DXCreateCubeTextureFromFile( d3dDevice, "Car/irr.dds", &carIrrMap );
    carSkyDome = dxPrimitiveCreateSphereMesh(256, 8, 8);
    isLoaded = true;
}

void SceneCar::render(ShaderGlobalSemantics &shaderSem, D3DXMATRIX &matView, D3DXMATRIX &matProjection) {
    
    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;

    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        dxCommitMatrix();
        carShader.setGlobalSemantics( &shaderSem );
        carShader.e()->SetTexture("cubeMapTexture", carCubeMap); 
        for(int a = carShader.begin("Skybox"); a < carShader.nPasses(); a++) {
            carShader.pass(a);
                dxSphere( carSkyDome );
            carShader.endPass();
        }
        carShader.end();
    dxPopMatrix();

    dxPushMatrix();
      dxRotateY( 90 );
      shaderSem.matWorld = dxGetMatrix();
      dxCommitMatrix();
      carShader.setGlobalSemantics( &shaderSem );
      carShader.e()->SetTexture("lightsTexture", carTextures[0]); 
      carShader.e()->SetTexture("interiorTexture", carTextures[1]); 
      carShader.e()->SetTexture("interiorNormalTexture", carTextures[10]);
      carShader.e()->SetTexture("blackTexture", carTextures[2]); 
      carShader.e()->SetTexture("underTexture", carTextures[3]); 
      carShader.e()->SetTexture("grilleTexture", carTextures[4]); 
      carShader.e()->SetTexture("tireTexture", carTextures[5]);
      carShader.e()->SetTexture("tireNormalTexture", carTextures[9]);
      carShader.e()->SetTexture("rimTexture", carTextures[6]);
      carShader.e()->SetTexture("rim2Texture", carTextures[7]);
      carShader.e()->SetTexture("bdrfTexture", carTextures[8]);
      carShader.e()->SetTexture("cubeMapTexture", carCubeMap);
      carShader.e()->SetTexture("irradianceTexture", carIrrMap);
      carModel.render( &carShader, &shaderSem, "Car" );
    dxPopMatrix();
}

void SceneCar::update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem ) {
    getSceneAnimationFrame( &animFrame, &animFrameTime, &animFrameLength );
    if( animFrameLength < 0.000001 ) animFrameLength = 0.000001; FOV = 90;
    double cameraPathTimeFactor = animFrameTime / animFrameLength;
    cameraPath[ animFrame ].setCamera( cameraPathTimeFactor * 0.95, matView, shaderSem );
}

void SceneCar::release() {
    carTextures.clear();
    carModel.release();
    carShader.release();
    carCubeMap->Release();
    carIrrMap->Release();
    carSkyDome->release();
    isLoaded = false;
}