/** \file SceneChrono.cpp
<h2> Chronosphere Scene </h2> <br>
Copyright (C) 2011 Xi Chen <br>

-------------------------------------------------------------------------<br>
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version. <br>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. <br>

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. <br>
-------------------------------------------------------------------------<br>

\brief Chronosphere Scene
\author Xi Chen hypernewbie@gmail.com
\date 15/11/2011

*/

#include "../GameHeader/SceneChrono.h"

void SceneChrono::load() {
    loadSceneAnimationPaths( "Chronosphere/anim.path" );
    chronoTextures.load(0, SCENECHRONO_NOISE_FILE);
    chronoTextures.load(1, SCENECHRONO_RINGMODELTEX_FILE);
    chronoTextures.load(2, SCENECHRONO_RINGMODELTEX2_FILE);
    chronoTextures.load(3, SCENECHRONO_RINGMODELTEX3_FILE);
    chronoTextures.load(4, SCENECHRONO_RINGMODELTEX4_FILE);
    chronoTextures.load(5, SCENECHRONO_ROOMTEX_FILE);
    chronoTextures.load(6, SCENECHRONO_TUBETEX0_FILE);
    chronoTextures.load(7, SCENECHRONO_TUBETEX1_FILE);
    chronoTextures.load(8, SCENECHRONO_TUBETEX2_FILE);

    chronoSphereShader.loadFile(SCENECHRONO_SHADER_FILE);
    sphere = dxPrimitiveCreateSphereMesh( 1.0, 24, 24 );
    D3DXCreateVolumeTextureFromFile( d3dDevice, SCENECHRONO_VOLUME_FILE, &cytoVirusVolume );
    D3DXCreateCubeTextureFromFile( d3dDevice, SCENECHRONO_CUBEMAP_FILE, &cytoVirusCubeMap );

    chronoRingShader.loadFile(SCENECHRONO_RINGSHADER_FILE);
    ring.load( SCENECHRONO_RINGMODEL_FILE );

    chronoRoomShader.loadFile(SCENECHRONO_ROOMSHADER_FILE);
    generatorRoom.load("ChronoSphere/generatorRoom.obj");

    tube = dxPrimitiveCreateTube( 5.0f, 15.0f, 32, 4 );

    time = 0.0f;
    isLoaded = true;
}

void SceneChrono::renderTube(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection, bool reverseOrder) {
    dxPushMatrix();
        dxRotateX( 90.0f );
        chronoRoomShader.e()->SetTexture( "tubeTexture", chronoTextures[6] ); dxTexture(0, chronoTextures[6]);
        chronoRoomShader.e()->SetTexture( "tube2Texture", chronoTextures[7] ); dxTexture(1, chronoTextures[7]);
        chronoRoomShader.e()->SetTexture( "tube3Texture", chronoTextures[8] ); dxTexture(2, chronoTextures[8]);
        chronoRoomShader.e()->SetFloat( "time", time * 5.0f );
        
        if( reverseOrder ) {
            dxPushMatrix();
                dxScale( 0.9, 0.9, 0.9 );
                dxCommitMatrix();
                shaderSem.matWorld = dxGetMatrix();
                chronoRoomShader.setGlobalSemantics( &shaderSem );
                for(int a = chronoRoomShader.begin("Tube2"); a < chronoRoomShader.nPasses(); a++) {
                    chronoRoomShader.pass(a);
                    dxTube( tube );
                    chronoRoomShader.endPass();
                }
                chronoRoomShader.end();
            dxPopMatrix();
        }

        dxCommitMatrix();
        shaderSem.matWorld = dxGetMatrix();
        chronoRoomShader.setGlobalSemantics( &shaderSem );
        for(int a = chronoRoomShader.begin("Tube"); a < chronoRoomShader.nPasses(); a++) {
            chronoRoomShader.pass(a);
            dxTube( tube );
            chronoRoomShader.endPass();
        }
        chronoRoomShader.end();
        
        if( !reverseOrder ) {
            dxScale( 0.9, 0.9, 0.9 );
            dxCommitMatrix();
            shaderSem.matWorld = dxGetMatrix();
            chronoRoomShader.setGlobalSemantics( &shaderSem );
            for(int a = chronoRoomShader.begin("Tube2"); a < chronoRoomShader.nPasses(); a++) {
                chronoRoomShader.pass(a);
                dxTube( tube );
                chronoRoomShader.endPass();
            }
            chronoRoomShader.end();
        }
        dxRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
    dxPopMatrix();
}

void SceneChrono::render(ShaderGlobalSemantics &shaderSem, 
        D3DXMATRIX &matView, D3DXMATRIX &matProjection) {

    shaderSem.matView = matView;
    shaderSem.matProj = matProjection;
    
    dxTexture(0, chronoTextures[5]);
    chronoRoomShader.e()->SetTexture( "AOTexture", chronoTextures[5] ); 
    dxPushMatrix();
        dxTranslate(0.0f, -2.0f, 0.0f);
        dxScale( 0.13, 0.13, 0.13 );
        dxPushMatrix();
            dxRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
            dxCommitMatrix();
            generatorRoom.flipX = false; generatorRoom.flipY = false;
            generatorRoom.render(&chronoRoomShader, &shaderSem, "Room");
        dxPopMatrix();
        dxPushMatrix();
            dxTranslate(0.18f, 0.0f, 0.0f);
            dxRenderState(D3DRS_CULLMODE, D3DCULL_CW);
            dxCommitMatrix();
            generatorRoom.flipX = true; generatorRoom.flipY = false;
            generatorRoom.render(&chronoRoomShader, &shaderSem, "Room");
        dxPopMatrix();
        dxPushMatrix();
            dxRenderState(D3DRS_CULLMODE, D3DCULL_CW);
            dxCommitMatrix();
            generatorRoom.flipX = false; generatorRoom.flipY = true;
            generatorRoom.render(&chronoRoomShader, &shaderSem, "Room");
        dxPopMatrix();
        dxPushMatrix();
            dxTranslate(0.18f, 0.0f, 0.0f);
            dxRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
            dxCommitMatrix();
            generatorRoom.flipX = true; generatorRoom.flipY = true;
            generatorRoom.render(&chronoRoomShader, &shaderSem, "Room");
        dxPopMatrix();
    dxPopMatrix();
    dxRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

    dxRenderState(D3DRS_CULLMODE, D3DCULL_CW);
    renderTube( shaderSem, matView, matProjection , false);
    
    for(int a = 0; a < SCENECHRONO_RINGS; a++) {
        dxPushMatrix();
            dxScale( 0.0055 + 0.002 * a, 
                     0.0055 + 0.002 * a, 
                     0.0055 + 0.002 * a );
            dxTexture( 0, chronoTextures[1] );
            dxRotateX( ringRotation[a].x );
            dxRotateY( ringRotation[a].y );
            dxRotateZ( ringRotation[a].z );
            dxCommitMatrix();
            chronoRingShader.e()->SetTexture( "colourTexture", chronoTextures[1] ); dxTexture(0, chronoTextures[1]);
            chronoRingShader.e()->SetTexture( "specMapTexture", chronoTextures[2] ); dxTexture(2, chronoTextures[2]);
            chronoRingShader.e()->SetTexture( "lightExemptionTexture", chronoTextures[3] ); dxTexture(3, chronoTextures[3]);
            chronoRingShader.e()->SetTexture( "AOTexture", chronoTextures[4] ); dxTexture(4, chronoTextures[4]);
            ring.render(&chronoRingShader, &shaderSem, "Ring"); 
        dxPopMatrix();
    }

    dxPushMatrix();
        shaderSem.matWorld = dxGetMatrix();
        chronoSphereShader.setGlobalSemantics( &shaderSem );
        dxCommitMatrix();
        chronoSphereShader.e()->SetTexture( "volumeTexture", cytoVirusVolume );
        chronoSphereShader.e()->SetTexture( "cubeMapTexture", cytoVirusCubeMap );
        chronoSphereShader.e()->SetTexture( "noiseTexture", chronoTextures[0] );
        chronoSphereShader.e()->SetFloat( "time", time * 2.5f );
        for(int a = chronoSphereShader.begin("Chronosphere"); a < chronoSphereShader.nPasses(); a++) {
            chronoSphereShader.pass(a);
            dxSphere( sphere );
            chronoSphereShader.endPass();
        }
        chronoSphereShader.end();
    dxPopMatrix();

    dxRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
    renderTube( shaderSem, matView, matProjection, true );

    ringRotation[0].x = 0.027f * time * 1100.0f;
    ringRotation[0].y = 0.032f * time * 1100.0f;
    ringRotation[0].z = 0.013f * time * 1100.0f;
    ringRotation[1].x = 0.029f * time * 1100.0f;
    ringRotation[1].y = 0.035f * time * 1100.0f;
    ringRotation[1].z = 0.015f * time * 1100.0f;
    ringRotation[2].x = 0.025f * time * 1100.0f;
    ringRotation[2].y = 0.030f * time * 1100.0f;
    ringRotation[2].z = 0.018f * time * 1100.0f;
}

void SceneChrono::update( D3DXMATRIX &matView, ShaderGlobalSemantics *shaderSem ) {
    getSceneAnimationFrame( &animFrame, &animFrameTime, &animFrameLength );
    if( animFrameLength < 0.000001 ) animFrameLength = 0.000001;
    double cameraPathTimeFactor = animFrameTime / animFrameLength;
    cameraPath[ animFrame ].setCamera( cameraPathTimeFactor , matView, shaderSem );
}

void SceneChrono::release() {
    chronoTextures.clear();
    chronoSphereShader.release();
    sphere->release();
    cytoVirusVolume->Release();
    cytoVirusCubeMap->Release();
    chronoRingShader.release();
    ring.release();
    chronoRoomShader.release();
    generatorRoom.release();
    tube->release();
    isLoaded = false;
}